﻿using System;
using System.IO;
using System.Text;
using System.Windows.Forms;
using KellermanSoftware.Common;
using KellermanSoftware.Common.Gui;
using KellermanSoftware.NetPasswordGenerator;
using KellermanSoftware.NetPasswordGeneratorDemo;
using Newtonsoft.Json;

namespace NET_Password_Generator_Demo
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        private void generatePassword_Click(object sender, EventArgs e)
        {
            try
            {
                Register register = new Register();
                PasswordGenerator generator = new PasswordGenerator(register.UserName, register.License);
                SetOptions(generator.Config);
                SetAdvancedOptions(generator.Config);
                StringBuilder sb = new StringBuilder((int)passwordsToGenerate.Value * generator.Config.MaxLength);
                for (int i = 0; i < passwordsToGenerate.Value; i++)
                {
                    sb.AppendLine(generator.Generate());
                }

                results.Text = sb.ToString();

            }
            catch (Exception ex)
            {
                GuiUtility.MessageBoxExclamation(ex.Message, "Could not generate passwords");
            }
        }

        private void frmMain_Load(object sender, EventArgs e)
        {
            SetDefaults();
        }

        private void SetDefaults()
        {
            PasswordConfig config = LoadConfig();

            LoadOptions(config);
            LoadAdvancedOptions(config);
        }

        private void LoadAdvancedOptions(PasswordConfig config)
        {
            GuiUtility.ListToCombo(wordStyle, ObjectUtil.EnumToList<WordStyle>());
            wordStyle.SelectedItem = config.WordStyle.ToString();

            GuiUtility.ListToCombo(numberPosition, ObjectUtil.EnumToList<Position>());
            numberPosition.SelectedItem = config.NumberPosition.ToString();

            GuiUtility.ListToCombo(specialCharacterPosition, ObjectUtil.EnumToList<Position>());
            specialCharacterPosition.SelectedItem = config.SpecialCharacterPosition.ToString();

            validSpecialCharacters.Text = config.ValidSpecialCharacters;
            minimumWords.Value = config.MinWords;
            maximumWords.Value = config.MaxWords;
            passwordsToGenerate.Value = 100;
        }

        private PasswordConfig LoadConfig()
        {
            string directoryPath =
                Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData),
                    "KellermanSoftware", "NetPasswordGeneratorDemo");

            if (!Directory.Exists(directoryPath))
            {
                Directory.CreateDirectory(directoryPath);
            }

            string filePath = Path.Combine(directoryPath, "PasswordConfig.json");

            if (File.Exists(filePath))
                return JsonConvert.DeserializeObject<PasswordConfig>(File.ReadAllText(filePath));

            return new PasswordConfig();
        }

        private void SaveConfig()
        {
            string directoryPath =
                Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData),
                    "KellermanSoftware", "NetPasswordGeneratorDemo");

            if (!Directory.Exists(directoryPath))
            {
                Directory.CreateDirectory(directoryPath);
            }

            string filePath = Path.Combine(directoryPath, "PasswordConfig.json");

            PasswordConfig config = new PasswordConfig();
            SetOptions(config);
            SetAdvancedOptions(config);

            File.WriteAllText(filePath, JsonConvert.SerializeObject(config, Formatting.Indented));
        }

        private void SetAdvancedOptions(PasswordConfig config)
        {
            config.WordStyle = (WordStyle) Enum.Parse(typeof(WordStyle), wordStyle.SelectedItem.ToString());
            config.NumberPosition = (Position) Enum.Parse(typeof(Position), numberPosition.SelectedItem.ToString());
            config.SpecialCharacterPosition =
                (Position) Enum.Parse(typeof(Position), specialCharacterPosition.SelectedItem.ToString());
            config.ValidSpecialCharacters = validSpecialCharacters.Text;
            config.MinWords = (int) minimumWords.Value;
            config.MaxWords = (int) maximumWords.Value;
        }

        private void SetOptions(PasswordConfig config)
        {
            config.MinLength = (int) minimumLength.Value;
            config.MaxLength = (int) maximumLength.Value;
            config.MinNumbers = (int) minimumNumbers.Value;
            config.MaxNumbers = (int) maximumNumbers.Value;
            config.MinSpecialCharacters = (int) minimumSpecialCharacters.Value;
            config.MaxSpecialCharacters = (int) maximumSpecialCharacters.Value;
            config.MinUpperLetter = (int) minimumUpperLetters.Value;
            config.MinLowerLetter = (int) minimumLowerLetters.Value;
        }

        private void LoadOptions(PasswordConfig config)
        {
            minimumLength.Value = config.MinLength;
            maximumLength.Value = config.MaxLength;
            minimumNumbers.Value = config.MinNumbers;
            maximumNumbers.Value = config.MaxNumbers;
            minimumSpecialCharacters.Value = config.MinSpecialCharacters;
            maximumSpecialCharacters.Value = config.MaxSpecialCharacters;
            minimumUpperLetters.Value = config.MinUpperLetter;
            minimumLowerLetters.Value = config.MinLowerLetter;
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            SaveConfig();
        }

        private void registerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmRegistration reg = new frmRegistration(new Register());
            reg.ShowDialog(this);
        }
    }
}
